/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef EXCEPTIONS_HPP
#define EXCEPTIONS_HPP

/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/


#include "config.h"
#include "gettext.h"
#define _(STRING)    gettext(STRING)

#include<string>
#include <exception>

using namespace std;


class ExceptionCancel : public std::exception {};
class ExceptionSupportNotCompiled : public std::exception {};
class ExceptionVersionMismatch : public std::exception {};


class ExceptionReadFile : public std::exception {};
class ExceptionDirOpen : public std::exception {};
class ExceptionReadDirEnd : public std::exception {};
class ExceptionFileIconNotFound: public std::exception {};
class ExceptionMustRecursive: public std::exception {};

class ExceptionNoDescription:  public std::exception {};

class ExceptionFileSystem: public std::exception
{
	public:
	
	virtual string getDescription(){return _("error");}
	
	~ExceptionFileSystem() throw() {}
	
	string getFileName() const
	{
		return fileName;
	}
	
	string getOperation() const
	{
		return operation;
	}
	
	protected:	
	string fileName;
	string operation;
};

class ExceptionAccess  : public ExceptionFileSystem 
{
public:
	ExceptionAccess(string _fileName, string _operation){fileName=_fileName;operation=_operation;}
	string getDescription(){return _("access error: ")+fileName;}
};

class ExceptionAlreadyExist  : public ExceptionFileSystem 
{
public:
	ExceptionAlreadyExist(string _fileName, string _operation){fileName=_fileName;operation=_operation;}
	string getDescription(){return _("file already exists: ")+fileName;}
};

class ExceptionNoEnoughSpace  : public ExceptionFileSystem 
{
public:
	ExceptionNoEnoughSpace(string _fileName, string _operation){fileName=_fileName;operation=_operation;}
	string getDescription(){return _("not enough space for file: ")+fileName;}

};

class ExceptionNoFile  : public ExceptionFileSystem 
{
public:
	ExceptionNoFile(string _fileName, string _operation){fileName=_fileName;operation=_operation;}
	string getDescription(){return _("file doesn't exist: ")+fileName;}

};

class ExceptionFileSystemOther  : public ExceptionFileSystem 
{
public:
	ExceptionFileSystemOther(string _fileName, string _operation){fileName=_fileName;operation=_operation;}
	string getDescription(){ return _("error on file: ")+fileName;}
};


class ExceptionBusy  : public ExceptionFileSystem 
{
public:
	ExceptionBusy(string _fileName, string _operation){fileName=_fileName;operation=_operation;}
	string getDescription(){return _("file is busy: ")+fileName;}

};

class ExceptionConnectionLost  : public ExceptionFileSystem 
{
public:
	ExceptionConnectionLost(string _fileName, string _operation){fileName=_fileName;operation=_operation;}
	string getDescription(){return _("connection lost: ")+fileName;}

};

class ExceptionConnectionRefused  : public ExceptionFileSystem 
{
public:
	ExceptionConnectionRefused(string _fileName, string _operation){fileName=_fileName;operation=_operation;}
	string getDescription(){return _("connection refused: ")+fileName;}

};

class ExceptionAuthenticationFailed  : public ExceptionFileSystem 
{
public:
	ExceptionAuthenticationFailed(string _fileName, string _operation){fileName=_fileName;operation=_operation;}
	string getDescription(){return _("authentication failed: ")+fileName;}

};

class ExceptionWithDescription  : public ExceptionFileSystem 
{
public:
	ExceptionWithDescription(string description){fileName=description;}
	string getDescription(){ return fileName;}
};

class ExceptionNotImplemented  : public ExceptionFileSystem 
{
public:
	string getDescription(){return _("not implemented");}
};

class ExceptionExecutingCommand  : public ExceptionFileSystem 
{
public:
	ExceptionExecutingCommand(string _operation){operation=_operation;}
	string getDescription(){ return _("error on command: ")+operation;}
};


#endif
