/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef FILE_H
#define FILE_H

#include <iostream>
#include <vector>
using namespace std;


#include <boost/filesystem/path.hpp>
using namespace boost::filesystem;


#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>


/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/
class File
{
public:
    File();
    ~File();    
    bool normal();
    bool hidden();
  
	bool operator==(const File &file) const
	{
		if(getPath()==file.getPath())
			return true;
		else
			return false;
	}

	void setName(const string& theValue)
	{
	    name = theValue;
	}
	

	string getName() const
	{
	    return name;
	}
	
	bool isDirectory( ) const
	{
		return (type&DIRECTORY);
	}
	
	void setDirectory( )
	{
		type|=DIRECTORY;		
	}

	bool isSymlink( ) const
	{
		return (type&SYMLINK);
	}

	void setSize(double theValue)
	{
	    size = theValue;
	}
	

	double getSize() const
	{
	    return size;
	}

	void setUid(const int& theValue)
	{
	    uid = theValue;
	}
	

	int getUid() const
	{
	    return uid;
	}

	void setGid(const int& theValue)
	{
	    gid = theValue;
	}
	

	int getGid() const
	{
	    return gid;
	}

	void setUser(const string& theValue)
	{
	    user = theValue;
	}
	

	string getUser() const
	{
	    return user;
	}

	void setGroup(const string& theValue)
	{
	    group = theValue;
	}
	

	string getGroup() const
	{
	    return group;
	}
	
	void setFields(const vector< string >& theValue)
	{
	    fields = theValue;
	}
	

	vector< string > getFields() const
	{
	    return fields;
	}

	void setParentDirectory(const path& theValue)
	{
	    parentDirectory = theValue;
	}
	

	path getParentDirectory() const
	{
	    return parentDirectory;
	}
	
	path getPath()const
	{
		return path(parentDirectory/name);
	}

	void setSymlink(const string& theValue)
	{
		type|=SYMLINK;
	    	symlink = theValue;
	}
	

	string getSymlink() const
	{
	    return symlink;
	}

	void setMime(const string& theValue)
	{
	    mime = theValue;
	}
	

	string getMime() const
	{
	    return mime;
	}
	
	bool isExecutable()
	{
		return (mode&S_IXUSR | mode&S_IXGRP | mode&S_IXOTH);
	}

	void setMode(const mode_t& theValue)
	{
	    mode = theValue;
	}
	
	
	bool isUserExecutable() const;
	bool isUserReadable() const;

	mode_t getMode() const
	{
	    return mode;
	}
	
	bool isOwnerExecutable()
	{
		return (mode&S_IXUSR);
	}
	
	void setOwnerExecutable()
	{
		mode=mode|S_IXUSR;
	}

	bool isOwnerReadable()
	{
		return (mode&S_IRUSR);
	}
	
	void setOwnerReadable()
	{
		mode=mode|S_IRUSR;
	}
	
	bool isOwnerWritable()
	{
		return (mode&S_IWUSR);
	}
	
	void setOwnerWritable()
	{
		mode=mode|S_IWUSR;
	}	
	
	bool isGroupReadable()
	{
		return (mode&S_IRGRP);
	}
	
	void setGroupReadable()
	{
		mode=mode|S_IRGRP;
	}
	
	
	bool isGroupWritable()
	{
		return (mode&S_IWGRP);
	}
	
	void setGroupWritable()
	{
		mode=mode|S_IWGRP;
	}

	bool isGroupExecutable()
	{
		return (mode&S_IXGRP);
	}
	
	void setGroupExecutable()
	{
		mode=mode|S_IXGRP;
	}	
	
	bool isOtherExecutable()
	{
		return (mode&S_IXOTH);
	}
	
	void setOtherExecutable()
	{
		mode=mode|S_IXOTH;
	}
	
	bool isOtherReadable()
	{
		return (mode&S_IROTH);
	}
	
	void setOtherReadable()
	{
		mode=mode|S_IROTH;
	}
	
	void setOtherWritable()
	{
		mode=mode|S_IWOTH;
	}
	
	bool isOtherWritable()
	{
		return (mode&S_IWOTH);
	}
	
	void setUID()
	{
		mode=mode|S_ISUID;
	}
	
	bool isUID()
	{
		return (mode&S_ISUID);
	}
	
	void setGID()
	{
		mode=mode|S_ISGID;
	}
	
	bool isGID()
	{
		return (mode&S_ISGID);
	}		
	
	void setSticky()
	{
		mode=mode|S_ISVTX;
	}
	
	bool isSticky()
	{
		return (mode&S_ISVTX);
	}

	void setAccessTime ( const time_t& theValue )
	{
		accessTime = theValue;
	}
	

	time_t getAccessTime() const
	{
		return accessTime;
	}

	void setModificationTime ( const time_t& theValue )
	{
		modificationTime = theValue;
	}
	

	time_t getModificationTime() const
	{
		return modificationTime;
	}

	void setCreationTime ( const time_t& theValue )
	{
		creationTime = theValue;
	}
	

	time_t getCreationTime() const
	{
		return creationTime;
	}
	
	bool isCharacterDevice( ) const
	{
		return (type&CHARDEV);
	}
	
	void setCharacterDevice( )
	{
		type|=CHARDEV;		
	}
	
	bool isBlockDevice( ) const
	{
		return (type&BLOCKDEV);
	}
	
	void setBlockDevice( )
	{
		type|=BLOCKDEV;		
	}
	
	bool isFifo( ) const
	{
		return (type&FIFO);
	}
	
	void setFifo( )
	{
		type|=FIFO;		
	}

	bool isSocket( ) const
	{
		return (type&SOCK);
	}
	
	void setSocket( )
	{
		type|=SOCK;		
	}

	void setUTF8Name ( const string& theValue )
	{
		UTF8Name = theValue;
		utf8=false;
	}
	

	string getUTF8Name() const
	{
		if(!utf8)
		return UTF8Name;
		else
		return name;
	}
	
	bool isUTF8Name() const
	{
		return utf8;
	}
	
private:
    string name;
    string UTF8Name;
    bool utf8;
    
    path parentDirectory;
    int type;
    double size;
    int uid;
    int gid;
    string user;
    string group;
    string symlink;
    vector<string>fields;
    string mime;
    
    time_t accessTime;
    time_t modificationTime;
    time_t creationTime;

    
    mode_t mode;


	enum
	{
	DIRECTORY   = 64,   /**< Directory item */
	EXECUTABLE  = 128,  /**< Executable item */
	SYMLINK     = 256,  /**< Symbolic linked item */
	CHARDEV     = 512,  /**< Character special item */
	BLOCKDEV    = 1024, /**< Block special item */
	FIFO        = 2048, /**< FIFO item */
	SOCK        = 4096  /**< Socket item */
	};
    
    
};

#endif
