/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef FILELIST_HPP
#define FILELIST_HPP

#include <string>
#include <vector>
#include <list>
using namespace std;

#include "FileIcon.hpp"
#include "FileListIcons.hpp"
#include "Vfs.hpp"

class ThreadCommand;


class FileList
{
public:

	virtual void openParentDir()=0;
	virtual void openIcon( FileIcon fileIcon )=0;
	virtual void openDirectory( path p)=0;
	virtual void reloadDirectory()=0;
	virtual vector<string> getDirectories(void)=0;
	virtual void runCommand(string name,string param="")=0;
	virtual vector<string> getToolbarCommands(string)=0;
	virtual vector<string> filterContexCommands(vector<string>)=0; 
	virtual Vfs* getVfs()=0;
	virtual FileListIcons* getFileListIcons()=0;
	virtual void setOpposite(FileList* theValue)=0;
	virtual void sort(int nr=-1)=0;
	virtual void textFieldExecute(string)=0;
	virtual void textFieldCommandSwitch(int)=0;
	virtual void clicked(void)=0;	
	virtual path getCurrentDirectory()=0;
	virtual FileList* getOpposite()=0;
	virtual void setDroppedFileNames(const vector< string >& theValue)=0;
	virtual bool isDragAndDrop()=0;
	virtual void setDragAndDrop(bool theValue)=0;
	virtual string getTextFieldData()=0;
	virtual string getVfsName()=0;
	virtual string getVfsType()=0;
	virtual FileList* getFileListSync()=0;
	virtual void setFileListSync ( FileList* theValue )=0;
	virtual void stopThreadProcessing(ThreadCommand *tc)=0;
	virtual void startThreadProcessing(ThreadCommand * tc)=0;
	virtual ~FileList(){}
	virtual void setCurrentDirectory ( const path& theValue )=0;
	virtual path getNewDirectory()=0;
	virtual void waitForThreads()=0;
	virtual vector< string >& getPathHistory()=0;
	virtual vector<string> getAvailableCommandsForSelectedFiles()=0;
	virtual path accessFile(path filePath)=0;
	virtual void releaseFile(path filePath, bool commit=true)=0;	
	virtual vector<path> getSelectedFileIconsPaths(void)=0;
	virtual vector<string> getSelectedFileIcons(void)=0;
	virtual FileIcon getCurrentFileIcon()=0;
	virtual FileIcon getFileIcon(string p)=0;
	virtual void addFileIcon(File file)=0;
	virtual void addFile(string p)=0;
	virtual void setFiles(vector<File> filesVec)=0;
	virtual void replaceFileIcon(FileIcon fileIcon)=0;
	virtual void removeFileIcon(string p)=0;
	virtual void removeFileIcons()=0;
	virtual void selectFileIcon(string p)=0;
	virtual vector<string>getAllFilesPaths()=0;
	virtual void refreshFileIcon(string p)=0;
	virtual void acquire()=0;
	virtual void release()=0;
	virtual bool isAcquired()=0;
	virtual void tick()=0;
};

#endif
