/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef FILELISTSYNC_HPP
#define FILELISTSYNC_HPP

#include "FileList.hpp"
#include "Vfs.hpp"

#include <boost/thread/recursive_mutex.hpp>
#include <boost/thread/mutex.hpp>
using namespace boost;


/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/
class FileListSync: public FileList
{
public:
    FileListSync(FileList *fileList):f(fileList){}
    	
    	void openParentDir();
	void openIcon( FileIcon fileIcon );
	void openDirectory( path p);
	void reloadDirectory();
	vector<string> getDirectories(void);
	void runCommand(string name,string param="");
	vector<string> getToolbarCommands(string); 
	vector<string> filterContexCommands(vector<string>);  
	Vfs* getVfs();
	FileListIcons* getFileListIcons();
	void setOpposite(FileList* theValue);
	void sort(int nr=-1);
	void textFieldExecute(string);
	void textFieldCommandSwitch(int);
	void clicked(void);
	path getCurrentDirectory();	
	FileList* getOpposite();
	void setDroppedFileNames(const vector< string >& theValue);
	bool isDragAndDrop();
	void setDragAndDrop(bool theValue);
	string getTextFieldData();
	string getVfsType();
	string getVfsName();
	void setFileListSync ( FileList* theValue );
	FileList* getFileListSync();
	void stopThreadProcessing(ThreadCommand *tc);
	void startThreadProcessing(ThreadCommand * tc);
	void setCurrentDirectory ( const path& theValue );
	path getNewDirectory();
	void waitForThreads();
	vector< string >& getPathHistory();
	vector<string> getAvailableCommandsForSelectedFiles();
	path accessFile(path filePath);
	void releaseFile(path filePath, bool commit=true);
	
	vector<path> getSelectedFileIconsPaths(void);
	vector<string> getSelectedFileIcons(void);
	FileIcon getCurrentFileIcon();
	FileIcon getFileIcon(string p);
	void addFileIcon(File file);
	void addFile(string p);
	void setFiles(vector<File> filesVec);
	void replaceFileIcon(FileIcon fileIcon);
	void removeFileIcon(string p);
	void removeFileIcons();
	void selectFileIcon(string p);
	vector<string>getAllFilesPaths();
	void refreshFileIcon(string p);
	void acquire();
	void release();
	bool isAcquired();
	void tick();
	
private:
	FileList *f;  
	boost::recursive_mutex muteX;  

};

#endif
