/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "FilesEngine.hpp"
#include "Exceptions.hpp"
#include "vfs/VfsLocal.hpp"

#include<boost/tokenizer.hpp>
using namespace boost;


#include <boost/lexical_cast.hpp>
using boost::lexical_cast;


#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>
#include <pwd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <grp.h>
#include <sys/statvfs.h>
#include <math.h>

path FilesEngine::getHomeDir( void )
{
struct passwd *pwd;
char *str;

   if((str=getenv("HOME"))!=NULL) return path(str);
	if((str=getenv("USER"))!=NULL || (str=getenv("LOGNAME"))!=NULL)
	{
		if((pwd=getpwnam(str))!=NULL) return pwd->pw_dir;
	}
    if((pwd=getpwuid(getuid()))!=NULL) return pwd->pw_dir;
  	return path().root_path();

}

string FilesEngine::getCurrentUser( )
{

char *str;

	if((str=getenv("USER"))!=NULL || (str=getenv("LOGNAME"))!=NULL)
	{
		return string(str);
	}
return "";
}




bool FilesEngine::programExist( string &programName )
{
	const char *name = "PATH";
	string paths= getenv(name);
	
	typedef boost::tokenizer<boost::char_separator<char> >  tokenizer;
  	boost::char_separator<char> sep(":");
  	tokenizer tokens(paths, sep);
	
	for (tokenizer::iterator tokIter = tokens.begin();tokIter != tokens.end(); ++tokIter)
   	{
   		path p=path(*tokIter)/path(programName);
   		if(exists(p))
   		return true;
   	}
   	
	return false;

}




vector<string> FilesEngine::getCurrentGroups( )
{

vector<string> groups;
	
	int ngroups;
	long ngroups_max;
	ngroups_max = sysconf(_SC_NGROUPS_MAX) + 1;
	gid_t *groupTab=new gid_t[ngroups_max];
	ngroups = getgroups(ngroups_max, groupTab);

	for(int i=0;i<ngroups;++i)
	{
		group *grp=getgrgid(groupTab[i]);
		groups.push_back(grp->gr_name);
	}

	delete groupTab;
return groups;	

}


void FilesEngine::runFile( path file )
{
	string cmd=string("\"")+file.string()+string("\"")+"&";
	system(cmd.c_str());
}



void FilesEngine::copy(path & source, path & destination)
{
	VfsLocal vfs;
	vfs.copy(&vfs,source,destination/source.leaf(),NULL,NULL);
}

bool FilesEngine::exists( path &p )
{

    struct stat fileStatus;
    return ::stat(p.string().c_str(),&fileStatus)==0;

}



string FilesEngine::expandPath( const string & path )
{
	if (path.length() == 0 || path[0] != '~')
      	return path;

	const char *pfx = NULL;
	string::size_type pos = path.find_first_of('/');
	
	if (path.length() == 1 || pos == 1)
	{
		pfx = getenv("HOME");
		if (!pfx)
		{
		// Punt. We're trying to expand ~/, but HOME isn't set
		struct passwd *pw = getpwuid(getuid());
		if (pw)
		pfx = pw->pw_dir;
		}
	}
	else
	{
		string user(path,1,(pos==string::npos) ? string::npos : pos-1);
		struct passwd *pw = getpwnam(user.c_str());
		if (pw)
		pfx = pw->pw_dir;
	}
	
	// if we failed to find an expansion, return the path unchanged.
	
	if (!pfx)
	return path;
	
	string result(pfx);
	
	if (pos == string::npos)
	return result;
	
	if (result.length() == 0 || result[result.length()-1] != '/')
	result += '/';
	
	result += path.substr(pos+1);
	
	return result;
}

string FilesEngine::printSize(double  k)
{
	
	string metric="B";
	bool flag=false;
	
	if ( k > 1024 )
	{
	flag=true;
		k /= 1024;
		metric = "KB";
		if ( k > 1024 )
		{
		k /= 1024;
		metric = "MB";
		}
			if ( k > 1024 )
			{
			k /= 1024;
			metric = "GB";
			}
	}
		
	string size=lexical_cast<string>(floor(k));
	
	if(flag)
	{
		string subsize=lexical_cast<string>(ceil((k-floor(k))*10));
		size+="."+subsize;
	}
	
	return size+=metric;
}

void FilesEngine::createDir(path & dir)
{
	VfsLocal vfs;
	vfs.createDir(dir);
}

void FilesEngine::remove(path & p)
{
	VfsLocal vfs;
	vfs.remove(p);
}



map< string, string > FilesEngine::parseParam(string param)
{
	map<string,string>params;
	
		typedef boost::tokenizer<boost::char_separator<char> >  tokenizer;
  		boost::char_separator<char> sep(":");
  		tokenizer tokens(param, sep);
	
		for (tokenizer::iterator tokIter = tokens.begin();tokIter != tokens.end(); ++tokIter)
		{
			string pair=*tokIter;
			int pos=pair.find("=");
			string name=pair.substr(0,pos);
			string value=pair.substr(pos+1,pair.length());
			params[name]=value;
		}
	return params;	
}






/*
void FilesEngine::copy( path &source, path &destination)
{	
	
	if (source.string() != destination.string())
    	{
    		int overwrite=Question::ask;
    		
    		if(status)
    		status->setTotalSize(getTotalSize(source));
    		
    		
		copyRecursive (source, destination, NULL,overwrite);

	}
	else
	{
		
	}
	
	if(status)
	{
	status->close();
	status=NULL;
	}
}
*/



/*
void FilesEngine::copyRecursive( path &source, path &destination, inodelist * inodes, int &overwrite )
{
	string srcFile=source.string();
	string dstFile=destination.string();
	
	struct stat status1, status2;

	// Source file or directory does not exist
	if (::lstat (srcFile.c_str(), &status1) != 0)
		throw std::exception();
	
	// If target is not a directory, remove it if allowed
	if (::lstat (dstFile.c_str(), &status2) == 0)
	{
		if (!S_ISDIR (status2.st_mode))
		{
		
			if(overwrite==Question::no || overwrite==Question::no_all)
			return;

			if(overwrite==Question::ask)
			overwrite=question->askOverwrite(dstFile);
			
			
			if(overwrite==Question::no || overwrite==Question::no_all)
			return;
			else
			//if(overwrite==Question::yes_all || overwrite==Question::yes )
			{
			
				if (::unlink (dstFile.c_str()) != 0)
					throw std::exception();
			}
			
			if(overwrite==Question::yes)
			overwrite=Question::ask;

		}
	}

	// Source is direcotory: copy recursively
	if (S_ISDIR (status1.st_mode))
	{
		copyDirectory (source, destination, status1, inodes,overwrite);
	}
	
	// Source is regular file: copy block by block
	else if (S_ISREG (status1.st_mode))
	{
		copyFile (source, destination);
	}
	
	// Source is fifo: make a new one
	else if (S_ISFIFO (status1.st_mode))
	{
		if(mkfifo (dstFile.c_str(), status1.st_mode)!=0)
			throw std::exception();
		
	}
	
	// Source is device: make a new one
	else if (S_ISBLK (status1.st_mode) || S_ISCHR (status1.st_mode) || S_ISSOCK (status1.st_mode))
	{
		if(mknod (dstFile.c_str(), status1.st_mode, status1.st_rdev)!=0)
			throw std::exception();
	}
	
	// Source is symbolic link: make a new one
	else if (S_ISLNK (status1.st_mode))
	{		
		if(::symlink (symlink (srcFile.c_str()).string().c_str(), dstFile.c_str())!=0)
			throw std::exception();	
	}


}

void FilesEngine::copyDirectory( path &source, path &destination, struct stat & parentstatus, inodelist * inodes, int &overwrite )
{	
	string srcFile=source.string();
	string dstFile=destination.string();
	
	struct stat status1;
	inodelist *in, inode;
	struct dirent *dp;
	DIR *dirp;
	
	// See if visited this inode already
	for (in = inodes; in; in = in->next)
	{
		if (in->st_ino == parentstatus.st_ino)
		return;
	}
	
	// Try make directory, if none exists yet
	if (mkdir (dstFile.c_str(), parentstatus.st_mode | S_IWUSR) != 0 && errno != EEXIST)
		throw std::exception();
	
	// Can we stat it
	if (::lstat (dstFile.c_str(), &status1) != 0 || !S_ISDIR (status1.st_mode))
		throw std::exception();
	
	// Try open directory to copy
	dirp = opendir (srcFile.c_str());
	if (!dirp)
		throw std::exception();
	
	// Add this to the list
	inode.st_ino = status1.st_ino;
	inode.next = inodes;
	
	
	while ((dp = readdir (dirp)) != NULL)
	{
		if (dp->d_name[0] != '.' || (dp->d_name[1] != '\0' && (dp->d_name[1] != '.' || dp->d_name[2] != '\0')))
		{	
			try
			{	
			path src=source/path(dp->d_name);
			path dst=destination/path(dp->d_name);	
			copyRecursive(src,dst,  &inode,overwrite);
			}
			catch(...)
			{
				closedir (dirp);
				throw std::exception();
			}
		}
	}	
	// Close directory
	closedir (dirp);

}
*/



string FilesEngine::convertTimeToString(time_t time)
{
	time_t tmp=time;
	struct tm tmresult;
	char buffer[512];
	int len=strftime(buffer,sizeof(buffer),"%m/%d/%Y %H:%M:%S",localtime_r(&tmp,&tmresult)); /*TODO*/
	string ret=string(buffer);
	ret.substr(0,len);
	return ret;
}


string FilesEngine::permissionsToString(mode_t mode)
{
	string result="----------";
	result[0]=(S_ISLNK(mode)) ? 'l' : (mode&S_IFREG) ? '-' : (mode&S_IFDIR) ? 'd' : (mode&S_IFCHR) ? 'c' : (mode&S_IFBLK) ? 'b' : (mode&S_IFIFO) ? 'p' : (mode&S_IFSOCK) ? 's' : '?';
	result[1]=(mode&S_IRUSR) ? 'r' : '-';
	result[2]=(mode&S_IWUSR) ? 'w' : '-';
	result[3]=(mode&S_ISUID) ? 's' : (mode&S_IXUSR) ? 'x' : '-';
	result[4]=(mode&S_IRGRP) ? 'r' : '-';
	result[5]=(mode&S_IWGRP) ? 'w' : '-';
	result[6]=(mode&S_ISGID) ? 's' : (mode&S_IXGRP) ? 'x' : '-';
	result[7]=(mode&S_IROTH) ? 'r' : '-';
	result[8]=(mode&S_IWOTH) ? 'w' : '-';
	result[9]=(mode&S_ISVTX) ? 't' : (mode&S_IXOTH) ? 'x' : '-';
	return result;
}

void FilesEngine::translateErrnoToException(string fileName, string operation)
{
	if(errno==EACCES || errno==EPERM || errno==EROFS)
		throw ExceptionAccess(fileName,operation);
	else if (errno==EEXIST)
		throw ExceptionAlreadyExist(fileName,operation);
	else if (errno==ENOSPC)
		throw ExceptionNoEnoughSpace(fileName,operation);
	else if (errno==EBUSY)
		throw ExceptionBusy(fileName,operation);
	else			
		throw ExceptionFileSystemOther(fileName,operation);
}

string FilesEngine::getEnv(string name)
{
	string value;
	char *charvalue=getenv(name.c_str());
	if(charvalue!=NULL)
	value=charvalue;
	else
	value="";
	
	return value;
}


mode_t FilesEngine::stringToPremissions(string perm)
{
	mode_t mode=0;
	
	char *p=const_cast<char *>(perm.c_str());
	
	/*TODO socket etc*/
	if(p[0]=='-')
	mode=mode|S_IFREG;
	else
	mode=mode|S_IFDIR;

	if(p[1]=='r')
	mode=mode|S_IRUSR;
	if(p[2]=='w')
	mode=mode|S_IWUSR;
	if(p[3]=='x')
	mode=mode|S_IXUSR;
	if(p[4]=='r')
	mode=mode|S_IRGRP;
	if(p[5]=='w')
	mode=mode|S_IWGRP;
	if(p[6]=='x')
	mode=mode|S_IXGRP;
	if(p[7]=='r')
	mode=mode|S_IROTH;
	if(p[8]=='w')
	mode=mode|S_IWOTH;
	if(p[9]=='x')
	mode=mode|S_IXOTH;
	
	if(p[3]=='s')
	mode=mode|S_ISUID;
	
	if(p[6]=='s')
	mode=mode|S_ISGID;

	if(p[9]=='t')
	mode=mode|S_ISVTX;
	
	return mode;
}

string FilesEngine::permissionsToOctetString(mode_t mode)
{
    int x = ( ( mode & S_IRUSR ) | ( mode & S_IWUSR ) | ( mode & S_IXUSR ) ) / 64;
    int y = ( ( mode & S_IRGRP ) | ( mode & S_IWGRP ) | ( mode & S_IXGRP ) ) / 8;
    int z = ( ( mode & S_IROTH ) | ( mode & S_IWOTH ) | ( mode & S_IXOTH ) );
    
    return lexical_cast<string>(x)+lexical_cast<string>(y)+lexical_cast<string>(z);
}

double FilesEngine::getFreeDiskSize(path p)
{
	struct statvfs64 stats;
	
	double size=0;
		
	if (statvfs64 (p.string().c_str (), &stats) != -1)
	{	
	size= stats.f_bsize *  stats.f_bavail;
	}
	
	return size;
}

double FilesEngine::getDiskSize(path p)
{
	struct statvfs64 stats;
	
	double size=0;
		
	if (statvfs64 (p.string().c_str (), &stats) != -1)
	{	
	size= stats.f_frsize *  stats.f_blocks;
	}
	
	return size;
}


