/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "Iconv.hpp"
#include "Settings.hpp"
#include <errno.h>

#include <libxml/parser.h>
#include <libxml/xpath.h>

#include "config.h"

Iconv::Iconv(string toEncoding,string fromEncoding)
{
	handle=iconv_open(toEncoding.c_str(), fromEncoding.c_str());
	
	if( handle == iconv_t(-1))
	{
		cout<<"error initializing iconv"<<endl;
	}

}

Iconv::~Iconv()
{
	iconv_close(handle);
}


string Iconv::encode(const string name)
{
	char *text=(char*)name.c_str();

	size_t inlen = ::strlen(text) + 1;
	size_t outlen = inlen*2;
	char *outbuf = new char[outlen];
	char* inbuf = text;
	::memset(outbuf,0,outlen);
	char *outbufp = outbuf;
	size_t r = iconv(handle,&inbuf,&inlen,&outbufp,&outlen);
	if(r < 0)
	{
		cout<<name<<endl;
			
			if(errno == E2BIG)
			cout<<"BIG"<<endl;
			else if(errno == EILSEQ)
			cout<<"EILSEQ"<<endl;
			else if(errno == EINVAL)
			cout<<"EINVAL"<<endl;
			
	
		delete[] outbuf;
		return "";
	}
	//delete[] text;
	//*text = outbuf;
	return string(outbuf);






/*
	const char *str=name.c_str();
	size_t len=name.length();
	
	if( !str || len == 0)
		return string();


	string       out;
	char         obuf[4097];
	char        *optr;
	size_t       olen;

	char        *sptr = (char *)str;
	size_t       slen = len;

	while( slen > 0)
	{
		obuf[0] = '\0';
		optr = (char *)obuf;
		olen = sizeof(obuf) - sizeof(obuf[0]);

		size_t ret = iconv(handle,&sptr, &slen, &optr, &olen);
		if( ret == size_t(-1))
		{
		
			cout<<name<<endl;
			
			if(errno == E2BIG)
			cout<<"BIG"<<endl;
			else if(errno == EILSEQ)
			cout<<"EILSEQ"<<endl;
			else if(errno == EINVAL)
			cout<<"EINVAL"<<endl;
			
			
			cout<<"EXCEPTION"<<endl;
			return "";
			
		}
		*optr = '\0';
		out  += obuf;
	}

	return out;
*/
}

/*
bool
Encoding::p_IsMultiByte(char c)
{
	bool b = c>>7;
	
	return !b;
}
*/


Iconv::Iconv()
{
	handle=iconv_open("UTF-8", singleton<Settings>::instance().getNonUTF8DefaultCharset().c_str() );
	
	if( handle == iconv_t(-1))
	{
		cout<<"error initializing iconv"<<endl;
	}
}

bool Iconv::isUTF8(const string name)
{
	#ifndef HAVE_LIBFOX_1_6
		return true;
	#endif
	
	return xmlCheckUTF8(( xmlChar *) name.c_str() );
}

string Iconv::fromUTF8(const string name)
{
	Iconv ic(singleton<Settings>::instance().getNonUTF8DefaultCharset().c_str(),"UTF-8");
	return ic.encode(name);
	
}

string Iconv::toUTF8(const string name)
{	
	Iconv ic("UTF-8",singleton<Settings>::instance().getNonUTF8DefaultCharset().c_str());
	return ic.encode(name);
}

bool Iconv::isRealyUTF8(const string name)
{
	return xmlCheckUTF8(( xmlChar *) name.c_str() );
}

string Iconv::convertToUTF8IfNeeded(const string name)
{
	if(!isUTF8(name))
	return toUTF8(name);
	else
	return name;
}