/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "VfsFactory.hpp"
#include "vfs/VfsLocal.hpp"
#include "FilesEngine.hpp"
#include "libfactory/Factorable.hpp"
#include "libfactory/ClassFactory.hpp"

#include "Exceptions.hpp"
#include "binreloc.h"


VfsFactory::VfsFactory()
{
	ClassFactory &factory=singleton<ClassFactory>::instance();
	
		
	if(FilesEngine::getEnv("NAO_VFS_PLUGINS_PATH")!="")
	{
	factory.addBaseDirectory( FilesEngine::getEnv("NAO_VFS_PLUGINS_PATH") );
	}
	
	if(FilesEngine::getEnv("NAO_COMMANDS_PLUGINS_PATH")!="")
	{
	factory.addBaseDirectory( FilesEngine::getEnv("NAO_COMMANDS_PLUGINS_PATH") );	
	}
	
	factory.addBaseDirectory(string(br_find_lib_dir(PATH_LIBDIR))+"/nao");

}


VfsFactory::~VfsFactory()
{
}

Vfs * VfsFactory::getVfs( string name )
{
	Vfs *vfs;
	if(name=="Local")
	{
		vfs=new VfsLocal;
	}
	else
	{
		/*TODO load from shared library*/		
		Settings& settings=singleton<Settings>::instance();	
		ClassFactory &factory=singleton<ClassFactory>::instance();
		try
		{
		vfs=dynamic_cast<Vfs*>(factory.createPure( "Vfs"+name ));
		}
		catch(FactoryCantCreate &e)
		{
			vfs=new VfsLocal;
			throw ExceptionSupportNotCompiled();
		}	
	}
	
	return vfs;	
	
}


