/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "FoxFileList.hpp"
#include "FoxShutter.hpp"
#include "FoxFileListItem.hpp"
#include "FoxInputDialog.hpp"
#include "FoxFonts.hpp"
#include "config.h"
#include "../FileListSync.hpp"
#include "Iconv.hpp"


#include <boost/lexical_cast.hpp>
using boost::lexical_cast;

#include <fxkeys.h>

FXDEFMAP (FoxFileList) FoxFileListMap[] =
{
    FXMAPFUNC (SEL_DOUBLECLICKED, FoxFileList::ID_FILE, FoxFileList::onOpenFile),
    FXMAPFUNC (SEL_CLICKED, FoxFileList::ID_FILE, FoxFileList::onClick),
    FXMAPFUNC (SEL_MOTION, 0, FoxFileList::onMotion),
    FXMAPFUNC (SEL_LEFTBUTTONPRESS, 0, FoxFileList::onMouseButtonPress),
    FXMAPFUNC (SEL_LEFTBUTTONRELEASE, 0, FoxFileList::onMouseButtonRelease),
    FXMAPFUNC (SEL_MIDDLEBUTTONPRESS, FoxFileList::ID_FILE, FoxFileList::onUpDir),
    FXMAPFUNCS (SEL_COMMAND, 1000,2000, FoxFileList::onCommand),
    FXMAPFUNC (SEL_FOCUSIN, FoxFileList::ID_FILE, FoxFileList::onFocus),
    FXMAPFUNC (SEL_RIGHTBUTTONPRESS, FoxFileList::ID_FILE, FoxFileList::onRightButtonPress),
    FXMAPFUNC (SEL_RIGHTBUTTONRELEASE, FoxFileList::ID_FILE, FoxFileList::onPopup),
    FXMAPFUNC (SEL_COMMAND, FoxFileList::ID_HEADER_CHANGE, FoxFileList::onSort),  
    FXMAPFUNC (SEL_COMMAND, FoxFileList::ID_SORT_CHANGE, FoxFileList::onSort),
    FXMAPFUNC (SEL_LEAVE, FoxFileList::ID_HEADER_CHANGE, FoxFileList::onCmdResizeHeader),  /*TODO*/
    FXMAPFUNC (SEL_DRAGGED, 0, FoxFileList::onDragged),
    FXMAPFUNC (SEL_DND_ENTER, 0, FoxFileList::onDNDEnter),
    FXMAPFUNC (SEL_DND_LEAVE, 0, FoxFileList::onDNDLeave),
    FXMAPFUNC (SEL_DND_DROP, 0, FoxFileList::onDNDDrop),
    FXMAPFUNC (SEL_DND_MOTION, 0, FoxFileList::onDNDMotion),
    FXMAPFUNC (SEL_DND_REQUEST, 0, FoxFileList::onDNDRequest),
    FXMAPFUNC (SEL_BEGINDRAG, 0, FoxFileList::onBeginDrag),
    FXMAPFUNC (SEL_ENDDRAG, 0, FoxFileList::onEndDrag),
    FXMAPFUNC (SEL_CLIPBOARD_REQUEST, 0, FoxFileList::onClipboardRequest),
    FXMAPFUNC (SEL_COMMAND, FoxFileList::ID_TEXT_FIELD, FoxFileList::onTextFieldExecute), 
    FXMAPFUNC (SEL_COMMAND, FoxFileList::ID_TEXT_FIELD_SWITCH, FoxFileList::onTextFieldSwitch),
    FXMAPFUNC (SEL_KEYPRESS, 0, FoxFileList::onFocusNext),
    FXMAPFUNC (SEL_TIMEOUT, FoxFileList::ID_TIMER, FoxFileList::onTimer ),
    FXMAPFUNC (SEL_COMMAND,  FoxFileList::ID_STOP_EXECUTION, FoxFileList::onStopExecution),
    FXMAPFUNC (SEL_TIMEOUT,  FoxFileList::ID_OPENTIMER, FoxFileList::onOpenTimer),
};

FXIMPLEMENT (FoxFileList, FXIconList, FoxFileListMap, ARRAYNUMBER (FoxFileListMap));


FoxFileList::FoxFileList(FXComposite * p,FoxDockContainer *_dock, Panel *_panel)
 : FXIconList (p, this, ID_FILE, LAYOUT_FILL_X | LAYOUT_FILL_Y | ICONLIST_EXTENDEDSELECT),
  panel(_panel),
  dock(_dock),
  settings(singleton<Settings>::instance()),
  foxIconManager(FoxSingleton<FoxIconManager>::instance(p->getApp())),
  processing(false),
  rotateAngel(1),
  mouseGestureActive(false),
  pressedRightButton(false),
  active(false),
  lastAnyFileSelected(false),
  delay(0),
  textField(NULL),
  popupMenu(NULL),
  sortpop(NULL),
  textFieldPop(NULL)
{

constructor=true;
	
	dropEnable();
	app=getApp();
	p_id=pthread_self();
	
	source = new FXIconSource ( app );
	
	guiSignal= new FXGUISignal (getApp(),&foxSingalReceiver);


	 sortpop = new FXPopup (this); 	
	 
	 fileListImpl=new FileListImpl(this, panel);
	 fileList=new FileListSync(fileListImpl);
	 
	 string fileListStyle=settings.getFileListStyle(fileList->getVfsName());
	 setView(fileListStyle);
		
	 FXHorizontalFrame *hframe = new FXHorizontalFrame ( p, LAYOUT_FILL_X , 0, 0, 0, 0, 0, 0, 0, 0 );
		
	 executionButton=new FXButton (hframe,"   ", NULL, this, ID_STOP_EXECUTION,BUTTON_TOOLBAR);	
	 statusInfo = new FXLabel (hframe, "",NULL,LAYOUT_FILL_X|JUSTIFY_LEFT|ICON_BEFORE_TEXT);
	 statusInfo->setBackColor(getApp()->getShadowColor ());
	 
	 getApp () ->addTimeout ( this, ID_TIMER, 500 );
	
	 settings.addSettingsObservator( this );	
	
	 
		
constructor=false;	

}




FoxFileList::~FoxFileList()
{



settings.removeSettingsObservator( this );	
getApp()->removeTimeout(this,ID_TIMER);
	delete fileList;
	delete fileListImpl;
	
	if (popupMenu)
  	delete popupMenu;
  	
  	if(sortpop)
  	delete sortpop;
  	
  	if(textFieldPop)
  	delete textFieldPop;
  	
  	//delete dragshell;
  	//delete dragshell2;

}


void FoxFileList::setTextFieldList( vector< string > list)
{
	return prepare<void>(boost::bind(&FoxFileList::setTextFieldListSync, this,list),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);
}
	
void FoxFileList::setTextFieldListSync( vector< string > list)
{	
	textFieldList=list;
	updateTextFieldList();
}

void FoxFileList::setTextFieldCommands( vector< string > commands )
{

	textFieldCommands=commands;
}

void FoxFileList::create( )
{
	FXIconList::create ();	
	
	if (!deleteType)
	{
		deleteType = getApp ()->registerDragType (deleteTypeName);
	}
	if (!urilistType)
	{
		urilistType = getApp ()->registerDragType (urilistTypeName);
	}	
	
	//dragshell->create();
	//dragshell2->create();


	
	 
	
	
}

long FoxFileList::onTextFieldExecute( FXObject * obj, FXSelector sel, void * ptr )
{
	string textFieldData=textField->getText().text();
	fileList->textFieldExecute(textFieldData);
}


long FoxFileList::onTextFieldSwitch( FXObject * obj, FXSelector sel, void * ptr )
{
	fileList->textFieldCommandSwitch(textFieldMenu->getCurrentNo ());
}


void FoxFileList::setHeaders( vector< Header > headers)
{
	return prepare<void>(boost::bind(&FoxFileList::setHeadersSync, this,headers),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);

}

void FoxFileList::setHeadersSync(vector< Header > headers )
{
	string name=_("name");
	name=_("size");
	name=_("owner");
	name=_("group");
	name=_("mode");
	name=_("mime");
	name=_("atime");
	name=_("mtime");
	name=_("ctime");
	name=_("path");
	name=_("relative_path");
	
	vector< Header >::iterator iter;
	for( iter=headers.begin();iter!=headers.end();++iter)
	{
	appendHeader (_(iter->name.c_str()), NULL, iter->width);
	new FXOption (sortpop, _(iter->name.c_str()), NULL, this, ID_SORT_CHANGE, FRAME_NONE | JUSTIFY_HZ_APART | ICON_AFTER_TEXT);
	}
}



bool FoxFileList::isFileIconSelected( void )
{
	return prepare<bool>(boost::bind(&FoxFileList::isFileIconSelectedSync, this),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL_BOOL,mutexS);
}



bool FoxFileList::isFileIconSelectedSync( void )
{

	int nr=getCurrentItem();
	
	if(nr!=-1)
	{
		if(isItemSelected(nr))
		return true;
	}
	

		for (int i = 0; i < getNumItems (); i++)
        	{
			if (isItemSelected (i))
			{
				return true;    	   	
			}
		}
		return false;

	/*
	
	if(nr==-1)
		return false;
	else
		if(isItemSelected(nr))
		return true;
		else
		return false;
	*/	
		
}



void FoxFileList::addFileIcon( FileIcon fileIcon )
{
	prepare<void>(boost::bind(&FoxFileList::addFileIconSync, this, fileIcon),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);
}

void FoxFileList::addFileIconSync(FileIcon fileIcon)
{

	FXIcon *icon=foxIconManager.get(fileIcon.getIconName());
	FXIcon *extraIcon=NULL;
	if(fileIcon.getExtraIconName().size())
        {
        	extraIcon=foxIconManager.get(fileIcon.getExtraIconName());
        }	
	
	FoxFileListItem *listItem=new FoxFileListItem(fileIcon,icon,icon,app,extraIcon);	
	
	
	appendItem (listItem);
	listItem->setDraggable(TRUE); 
}

void FoxFileList::addFileIcons(vector< FileIcon > fileIcons)
{
	prepare<void>(boost::bind(&FoxFileList::addFileIconsSync, this, fileIcons),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);
}

void FoxFileList::addFileIconsSync(vector< FileIcon > fileIcons)
{
	vector< FileIcon >::iterator iter;
	for(iter=fileIcons.begin();iter!=fileIcons.end();++iter)
	{
		addFileIconSync(*iter);
	}
}








void FoxFileList::clearItems( void )
{
	prepare<void>(boost::bind(&FoxFileList::clearItemsSync, this),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);
}

void FoxFileList::clearItemsSync( void )
{
	FXIconList::clearItems();
}





void FoxFileList::openPath( string p )
{

	fileList->openDirectory(path(p));
}

vector< string > FoxFileList::getDirectories( void )
{
	return fileList->getDirectories();
}



void FoxFileList::hideToolbars()
{
	//toolbar->hide();
	//toolbar2->hide();
	
}

void FoxFileList::showToolbars( )
{	
	
	makeToolbars();
		
		/*
	toolbar->dock(dock->getRightdock());
	//dock->getTopdock()->dockToolBar(toolbar2,0,0);
	toolbar2->dock(dock->getTopdock());
	dock->getTopdock()->dockToolBar(toolbar2,0,0);


	
	toolbar->show();
	toolbar->layout();
	toolbar->repaint();
	
	//toolbar->recalc();
		 
	toolbar2->show();
	//toolbar2->recalc();
	toolbar2->layout();
	toolbar2->repaint(); 
	*/
	
}





void FoxFileList::lostFocus( )
{
	if(!active)
	return;
	
	//toolbar->dock(docksite);
	//toolbar2->dock(docksite);
	//hideToolbars();
	
	deleteToolbars();
	
	statusInfo->setBackColor(getApp()->getShadowColor ());
	active=false;

}

void FoxFileList::makeToolbars( )
{
	deleteToolbars();
	
	
	commandsVector.clear();
	
	
	
	textField=new FXComboBox (dock->getToolbar1(), 30,this,ID_TEXT_FIELD,COMBOBOX_REPLACE|FRAME_THICK);	 
	textFieldPop = new FXPopup (dock->getToolbar1());  
	 
	
	

	textFieldMenu=new FXOptionMenu (dock->getToolbar1(), textFieldPop, LAYOUT_TOP |FRAME_THICK | JUSTIFY_HZ_APART | ICON_AFTER_TEXT);
	textFieldMenu->create();


	sortMenu=new FXOptionMenu (dock->getToolbar1(), sortpop, LAYOUT_TOP |FRAME_THICK | JUSTIFY_HZ_APART | ICON_AFTER_TEXT);
	sortMenu->create();
	
	
	
	makeToolbar("toolbar1");
	makeToolbar("toolbar2");

	
	
	 	
	updateTextFieldList();
	textField->create();	
		
	vector<string>::iterator iter;

	for(iter=textFieldCommands.begin();iter!=textFieldCommands.end();++iter)
	{
		FXOption *option=new FXOption (textFieldPop, string(string("\t")+settings.getCommand(*iter).getDescription(settings.getLanguage())).c_str(),foxIconManager.get( settings.getCommandIcon(*iter)) , this, ID_TEXT_FIELD_SWITCH, FRAME_NONE | JUSTIFY_HZ_APART | ICON_AFTER_TEXT,0,0,0,0,0,0,0,0);	
	}
	
	textFieldPop->create();
	
	dock->getToolbar1()->recalc();
	dock->getToolbar2()->recalc();
   	
}

void FoxFileList::deleteToolbars( )
{	

	if(textField==NULL)
	return;



	//delete sortpop;
	delete textField;
	textField=NULL;
	
	
	delete textFieldPop;
	textFieldPop=NULL;
	delete textFieldMenu;
	textFieldMenu=NULL;
	delete sortMenu;
	sortMenu=NULL;

	vector<FXButton*>::iterator iter;
	for(iter=buttons.begin() ;iter!=buttons.end() ;++iter )
	{
		delete *iter;
	}			
	buttons.clear();

}



void FoxFileList::makeToolbar( string name )
{
	vector<string>commands=fileList->filterContexCommands(fileList->getToolbarCommands( name ));
	vector<string>::iterator iter;

	FXToolBar*toolb;
	if(name=="toolbar2")
	toolb=dock->getToolbar1();
	else
	toolb=dock->getToolbar2();
	
	for(iter=commands.begin();iter!=commands.end();++iter)
	{
		CommandInfo cmd=settings.getCommand(*iter);
		FXButton *button=new FXButton (toolb,string( "\t"+cmd.getDescription(settings.getLanguage())).c_str(), foxIconManager.get(cmd.getIconName()), this, settings.getCommand(*iter).getNr(), FRAME_RAISED | LAYOUT_TOP | LAYOUT_LEFT | BUTTON_TOOLBAR, 0, 0, 0, 0, 0, 0, 0, 0);
   		buttons.push_back(button);
   		commandsVector.push_back(*iter);
   		if(!constructor)
   		button->create();
   	}
 }  	

void FoxFileList::setOpposite( FoxFileList * theValue )
{
	opposite = theValue;
	
		if(opposite)
		{
			//if(hasFocus())
			//lostFocus();
			//opposite->lostFocus();
			fileList->setOpposite(opposite->getFileList());
		}
		else
		{
			fileList->setOpposite(NULL);
			//if(hasFocus())
			//lostFocus();
		}

}


void FoxFileList::setStatusInfo( string text, string iconName)
{
	prepare<void>(boost::bind(&FoxFileList::setStatusInfoSync, this, text,iconName),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);	
}

void FoxFileList::setStatusInfoSync(string text, string iconName)
{
		if(!Iconv::isUTF8(text))
		{		
			text=Iconv::toUTF8(text);
		}
		

	statusInfo->setText(text.c_str());
	statusInfo->setTipText(text.c_str());
	
	if(iconName=="")
	statusInfo->setIcon(NULL);
	else
	{
	FXIcon *icon=foxIconManager.get(iconName);
	//icon->resize(22,22);
	statusInfo->setIcon(icon);
	}
}


void FoxFileList::notify( )
{
	prepare<void>(boost::bind(&FoxFileList::notifySync, this),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);
}

void FoxFileList::notifySync( )
{

//	if(dock->getToolbar1()->shown())
//	showToolbars();

	
}




vector< FileIcon > FoxFileList::getSelectedFileIcons()
{
	return prepare<vector<FileIcon> >(boost::bind(&FoxFileList::getSelectedFileIconsSync, this),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL_VECTOR_FILEICON,mutexS);
}

vector< FileIcon > FoxFileList::getSelectedFileIconsSync()
{

	vector <FileIcon> fileIcons;
	for (int i = 0; i < getNumItems (); i++)
        {
        	if (isItemSelected (i))
          	{
			FoxFileListItem* listItem=dynamic_cast<FoxFileListItem*>(getItem (i));
			fileIcons.push_back(listItem->getFileIcon());        	   	
		}
	}

	return fileIcons;

	
}



Status * FoxFileList::getStatusPanel( Cancelable *cancelable )
{
	FoxStatus *status= new FoxStatus(this, cancelable);
	status->create();
	status->show( PLACEMENT_OWNER );
	statusVector.push_back(status);
	
	
	return status;
}



void FoxFileList::setShorcuts()
{

	FXAccelTable *table = getShell ()->getAccelTable ();
	vector<string> commands=settings.getAllCommands();
	vector<string>::iterator iter;
	for(iter=commands.begin();iter!=commands.end();++iter)
	{
		CommandInfo cmd=settings.getCommand(*iter);
	            
		if(cmd.getKey()=="Tab")
			tabCommandNr=cmd.getNr();
				
		string shortcut=cmd.getKeyMask();
		if(shortcut!="")
			shortcut+="+";
			
		shortcut+=cmd.getKey();
		if(cmd.getKey()!="")
		{
		#ifdef  HAVE_LIBFOX_1_6	
			/*TODO*/
			table->addAccel ( parseAccel(shortcut.c_str()), this, FXSEL (SEL_COMMAND, cmd.getNr()));
		#else
			table->addAccel ( fxparseAccel(shortcut.c_str()), this, FXSEL (SEL_COMMAND, cmd.getNr()));
		#endif
		}
		
	
	}
           
}




void FoxFileList::setView( string fileListStyle )
{
	prepare<void>(boost::bind(&FoxFileList::setViewSync, this, fileListStyle),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);
}

void FoxFileList::setViewSync( string fileListStyle )
{

FoxFonts &foxFonts=FoxSingleton<FoxFonts>::instance(app);

	if (fileListStyle == "big icons")
	{
		this->setListStyle (ICONLIST_EXTENDEDSELECT | ICONLIST_BIG_ICONS | ICONLIST_COLUMNS);
		setFont (foxFonts.getFont("big"));
	}
	else if (fileListStyle == "small icons")
	{
		this->setListStyle (ICONLIST_EXTENDEDSELECT | ICONLIST_MINI_ICONS | ICONLIST_COLUMNS);
		setFont (foxFonts.getFont("normal"));
	}
	else if (fileListStyle == "details")
	{
		this->setListStyle (ICONLIST_EXTENDEDSELECT | ICONLIST_DETAILED | ICONLIST_COLUMNS);
		setFont (foxFonts.getFont("normal"));
	}
	layout();
	//recalc();
	//update();
	repaint();
	//forceRefresh();

}


void FoxFileList::switchPanelsView( void )
{
	prepare<void>(boost::bind(&FoxFileList::switchPanelsViewSync, this),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);
}

void FoxFileList::switchPanelsViewSync(void )
{
	if(!settings.isSinglePanel())
	{
		panel->setPanelWidth(-1);
	}
	else
	{
		opposite->getPanel()->setPanelWidth(0);
	}
}



void FoxFileList::setFocus( )
{
	prepare<void>(boost::bind(&FoxFileList::setFocusSync, this),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);
}

void FoxFileList::setFocusSync()
{
	FXIconList::setFocus();
}


void FoxFileList::selectAll()
{
	prepare<void>(boost::bind(&FoxFileList::selectAllSync, this),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);
}

void FoxFileList::selectAllSync()
{
	for (int u = 0; u < getNumItems (); u++)
	{
		selectItem(u);
	}
}



void FoxFileList::startProcessing()
{
prepare<void>(boost::bind(&FoxFileList::startProcessingSync, this),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);	
}

void FoxFileList::stopProcessing()
{
prepare<void>(boost::bind(&FoxFileList::stopProcessingSync, this),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);	
}

void FoxFileList::startProcessingSync()
{
	processing=true;
}

void FoxFileList::stopProcessingSync()
{
	processing=false;
	executionButton->enable();	
}






















/**DRAG AND DROP****************************************************/



// Handle drag-and-drop enter, remember current directory
long FoxFileList::onDNDEnter (FXObject * sender, FXSelector sel, void *ptr)
{

    FXIconList::onDNDEnter (sender, sel, ptr);
    //orgdirectory=getDirectory();
    return 1;
}


// Handle drag-and-drop leave, restore current directory prior to drag
long FoxFileList::onDNDLeave (FXObject * sender, FXSelector sel, void *ptr)
{

    FXIconList::onDNDLeave (sender, sel, ptr);
    // getApp()->removeTimeout(this,ID_OPENTIMER);
    stopAutoScroll ();
    // setDirectory(orgdirectory);
    return 1;
}


// Handle drag-and-drop motion
long FoxFileList::onDNDMotion (FXObject * sender, FXSelector sel, void *ptr)
{

    FXEvent *event = (FXEvent *) ptr;
    FXint index = -1;

    // Cancel open up timer
    // getApp()->removeTimeout(this,ID_OPENTIMER);

    // Start autoscrolling
    if (startAutoScroll (event, FALSE))
    	return 1;

    // Give base class a shot
    if (FXIconList::onDNDMotion (sender, sel, ptr))
    	return 1;

    // Dropping list of filenames
    if (offeredDNDType (FROM_DRAGNDROP, urilistType))
    {
    
	dropaction = inquireDNDAction ();

    // Drop in the background
    //  dropdirectory=getDirectory();

    // What is being done (move,copy,link)


    // We will open up a folder if we can hover over it for a while
    index=getItemAt(event->win_x,event->win_y);
    
    if(index!=-1)
    {
    FoxFileListItem* listItem=dynamic_cast<FoxFileListItem*>(getItem (index));
    
	if( listItem->getFileIcon().getFile().isDirectory())
	{
		
		//fileList->openIcon(listItem->getFileIcon());
	// Set open up timer
	 getApp()->addTimeout(this,ID_OPENTIMER,700);
	
	// Directory where to drop, or directory to open up
	//dropdirectory=getItemPathname(index);
	}
      
     }

    // See if dropdirectory is writable
    // if(FXFile::isWritable(dropdirectory)){
    //  FXTRACE((100,"accepting drop on %s\n",dropdirectory.text()));
    acceptDrop (DRAG_ACCEPT);
    //   }
    return 1;
    }
    return 0;
}

long FoxFileList::onOpenTimer(FXObject*,FXSelector,void*)
{
  FXint xx,yy,index; FXuint buttons;
  getCursorPosition(xx,yy,buttons);
  index=getItemAt(xx,yy);
   if(index!=-1)
    {
    FoxFileListItem* listItem=dynamic_cast<FoxFileListItem*>(getItem (index));
    
	if( listItem->getFileIcon().getFile().isDirectory())
	{
		fileList->openIcon(listItem->getFileIcon());
	}
}	
  
  /*
  if(0<=index && isItemDirectory(index)){
    dropdirectory=getItemPathname(index);
    setDirectory(dropdirectory);
    getApp()->addTimeout(this,ID_OPENTIMER,700);
    }
  return 1;
  */
  }


// Handle drag-and-drop drop
long FoxFileList::onDNDDrop (FXObject * sender, FXSelector sel, void *ptr)
{

    // Cancel open up timer
//  getApp()->removeTimeout(this,ID_OPENTIMER);

    // Stop scrolling
    stopAutoScroll ();


    // Perhaps target wants to deal with it
    if (FXIconList::onDNDDrop (sender, sel, ptr))
    	return 1;
    dropData (false);


    return 0;
}


// Somebody wants our dragged data
long FoxFileList::onDNDRequest (FXObject * sender, FXSelector sel, void *ptr)
{

    FXEvent *event = (FXEvent *) ptr;
    FXuchar *data;
    FXuint len;

    // Perhaps the target wants to supply its own data
    if (FXIconList::onDNDRequest (sender, sel, ptr))
    return 1;

    // Return list of filenames as a uri-list
    if (event->target == urilistType)
    {
	if (!dragfiles.empty ())
	{
		len = dragfiles.length ();
		FXMEMDUP (&data, dragfiles.text (), FXuchar, len);
		setDNDData (FROM_DRAGNDROP, event->target, data, len);
	}
    return 1;
    }

    // Delete selected files
    if (event->target == deleteType)
    {
    	
    	/*FIXME*/
    	/*
    	for (int i = 0; i < getNumItems (); i++)
	{
		if (isItemSelected (i))
		{
			removeFileIcon(i);
		}
	}	
    	*/
	//FXTRACE ((100, "Delete files not yet implemented\n"));
	return 1;
    }

    return 0;
}

// Start a drag operation
long FoxFileList::onBeginDrag (FXObject * sender, FXSelector sel, void *ptr)
{
    if (FXIconList::onBeginDrag (sender, sel, ptr))
    	return 1;
    
    if (beginDrag (&urilistType, 1))
    {
	setDraggedFiles();	
	return 1;
    }
    return 0;
}


void FoxFileList::setDraggedFiles()
{

dragfiles = FXString::null;
	
	for (int i = 0; i < getNumItems (); i++)
	{
		if (isItemSelected (i))
		{
	
		FoxFileListItem *listItem = dynamic_cast<FoxFileListItem *>( getItem (i) );
		/*FIXME*/
		string path=listItem->getFileIcon().getFile().getPath().string();
		string name=listItem->getFileIcon().getFile().getName();
	
			if (name != "." && name != "..")
			{
			if (!dragfiles.empty ())
				dragfiles += "\r\n";
			dragfiles += FXURL::fileToURL (path.c_str ());
			}
		}
	}

}



void FoxFileList::clipboardCopy(bool cut)
{
	prepare<void>(boost::bind(&FoxFileList::clipboardCopySync, this,cut),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);

}

void FoxFileList::clipboardCopySync(bool cut)
{
	FXDragType types[1];
	types[0] = urilistType;
	
//	FXDragAction dropaction;
	
	if (acquireClipboard (types, 1))
	{	
		if(!cut)
		{
		settings.setCut(false);
		//dropaction=DRAG_COPY;
		}
		else
		{	
		settings.setCut(true);
		//dropaction=DRAG_MOVE;
		}	
		
		//FXEvent *event = (FXEvent *) ptr;
		//handleDrag (0, 0, dropaction);
	
		//filelist_opposite->dropaction=dropaction;		
		setDraggedFiles();
          }
}




void FoxFileList::clipboardPaste()
{
	prepare<void>(boost::bind(&FoxFileList::clipboardPasteSync, this),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);
 
}

void FoxFileList::clipboardPasteSync()
{
	dropData (true);
}



// End drag operation
long FoxFileList::onEndDrag (FXObject * sender, FXSelector sel, void *ptr)
{
    if (FXIconList::onEndDrag (sender, sel, ptr))
    return 1;
    endDrag ((didAccept () != DRAG_REJECT));
    setDragCursor (getDefaultCursor ());
    dragfiles = FXString::null;
    return 1;
}


// Dragged stuff around
long FoxFileList::onDragged (FXObject * sender, FXSelector sel, void *ptr)
{

    FXEvent *event = (FXEvent *) ptr;
    FXDragAction action;
    if (FXIconList::onDragged (sender, sel, ptr))
    return 1;
    action = DRAG_MOVE;
    if (event->state & CONTROLMASK)
    {action = DRAG_COPY;}
    if (event->state & SHIFTMASK)
    action = DRAG_MOVE;
    if (event->state & ALTMASK)
    action = DRAG_LINK;
    handleDrag (event->root_x, event->root_y, action);
    if (didAccept () != DRAG_REJECT)
    {
    if (action == DRAG_MOVE)
        setDragCursor (getApp ()->getDefaultCursor (DEF_DNDMOVE_CURSOR));
    else if (action == DRAG_LINK)
        setDragCursor (getApp ()->getDefaultCursor (DEF_DNDLINK_CURSOR));
    else
        setDragCursor (getApp ()->getDefaultCursor (DEF_DNDCOPY_CURSOR));
    }
    else
    {
    setDragCursor (getApp ()->getDefaultCursor (DEF_DNDSTOP_CURSOR));
    }
    return 1;
}





//----------------------------------------------------  


// We now really do have the clipboard, keep clipped text
long FoxFileList::onClipboardGained (FXObject * sender, FXSelector sel, void *ptr)
{
    FXIconList::onClipboardGained (sender, sel, ptr);
    return 1;
}


// We lost the clipboard, free clipped text
long FoxFileList::onClipboardLost (FXObject * sender, FXSelector sel, void *ptr)
{
    FXIconList::onClipboardLost (sender, sel, ptr);
    dragfiles=FXString::null;
    //clipped.clear();
    return 1;
}


// Somebody wants our clipped text
long FoxFileList::onClipboardRequest (FXObject * sender, FXSelector sel, void *ptr)
{
    FXEvent *event = (FXEvent *) ptr;
    FXuchar *data;
    FXuint len;
    len = dragfiles.length ();
    FXMEMDUP (&data, dragfiles.text (), FXuchar, len);
    setDNDData (FROM_CLIPBOARD, event->target, data, len);
    return 0;
}


void FoxFileList::dropData (bool clipboard )
{

	


	FXuchar *data;
	FXuint len;
	FXDNDOrigin origin;
	
	if (clipboard)
	{
		origin = FROM_CLIPBOARD;
		//dropaction = inquireDNDAction ();
		/*FIXME add cut*/
		if(settings.isCut())
		dropaction = DRAG_MOVE;
		else
		dropaction = DRAG_COPY;
		
		settings.setCut(false);
	}
	else
		origin = FROM_DRAGNDROP;
	
	// Get uri-list of files being dropped
	if (getDNDData (origin, urilistType, data, len))
	{
		
		FXRESIZE (&data, FXuchar, len + 1);
		data[len] = '\0';
		FXchar *p, *q;
		p = q = (FXchar *) data;
		
		vector < string > fileNames;
		
		while (*p)
		{
			while (*q && *q != '\r')
				q++;
			
			FXString url (p, q - p);
			FXString fileName (FXURL::fileFromURL (url));
			fileNames.push_back (fileName.text ());
			if (*q == '\r')
				q += 2;
			p = q;
			
		}
	
		fileList->setDroppedFileNames(fileNames);
		
		if(dropaction==DRAG_COPY)
		{
		fileList->runCommand( "Copy" );	
		}
		else if(dropaction==DRAG_MOVE)
		{
		fileList->runCommand( "Move" );
		FXuchar *data;
		FXuint len;
			getDNDData (origin, deleteType,data,len);	
		}
		FXFREE (&data);	
	}
}

/**DRAG AND DROP END****************************************************/


/**EVENT HANDLERS*******************************************************/

long FoxFileList::onUpDir( FXObject * obj, FXSelector sel, void * ptr )
{
	
	fileList->openParentDir();
}

long FoxFileList::onCommand( FXObject * obj, FXSelector sel, void * ptr )
{
	
	FXushort id = FXSELID (sel);
	fileList->runCommand( settings.getCommand(id).getName() );
	
	if (popupMenu)
    	popupMenu->hide();
    	
    	return 1;
	
	
}

long FoxFileList::onFocus( FXObject * obj, FXSelector sel, void * ptr )
{
	if(active)
	return 0;


	showToolbars();
	statusInfo->setBackColor(getApp()->getBaseColor ());
	setShorcuts();
	
	if(opposite)
	{
	opposite->lostFocus();
	}
	
	active=true;
	

	
	return 1;

}


long FoxFileList::onClick( FXObject * obj, FXSelector sel, void * ptr )
{
	
	fileList->getFileListSync()->clicked();
	
	return 1;
}

long FoxFileList::onPopup( FXObject * obj, FXSelector sel, void * ptr )
{

	pressedRightButton=false;
	
	if(mouseGestureActive)
	{
	mouseGestureActive=false;
	string gestureString;
	
	vector<int>::iterator intIter;
	for(intIter=gests.begin();intIter!=gests.end();++intIter)
	{
		gestureString+=lexical_cast<string>(*intIter);
	}
	
	vector<string> commands=settings.getAllCommands();
	vector<string>::iterator iter;
	for(iter=commands.begin();iter!=commands.end();++iter)
	{
		CommandInfo cmd=settings.getCommand(*iter);
		if(cmd.getGesture()==gestureString)
		{
		fileList->runCommand(*iter);
		break;
		}
	}
	
	
	gests.clear();
	
	
	return 0;
	}

	if (popupMenu)
    	{
    	delete popupMenu;
    	popupMenu = NULL;
    	}

	

	int num = 0;
	
	for (int u = 0; u < getNumItems (); u++)
	if (isItemSelected (u))
		num++;
	
	if (num <= 1)
	{
			FXint x, y;
			FXuint state;
			getCursorPosition (x, y, state);
			
			int item = getItemAt (x, y);
		
		if (getCurrentItem () >= 0)
			deselectItem (getCurrentItem ());
		
		if (item >= 0)
		{
			setCurrentItem (item);
			if (!selectItem (item));
		}
	}


	popupMenu = new FXMenuPane (this,FRAME_NONE);
	FXShutter *foxShutter = new FoxShutter (popupMenu, this);
	
	vector<string>shutters=settings.getShutters();
	vector<string>::iterator iter;
	int maxLabelWidth=0;
	
	string defaultShutter=settings.getDefaultShutter();
	int defaultShutterNum=0;
	int counter=0;
	for(iter=shutters.begin();iter!=shutters.end();++iter)
	{
	
		if(*iter=="special")
		{
			if(fileList->getAvailableCommandsForSelectedFiles().size()==0)
			continue;	
		}
	
	FXShutterItem*shutterItem = new FXShutterItem (foxShutter, iter->c_str(), foxIconManager.get("execute"), FRAME_NONE | LAYOUT_FILL_X | LAYOUT_FILL_Y | LAYOUT_TOP | LAYOUT_LEFT, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
	FXButton *but = shutterItem->getButton ();
        but->setButtonStyle (BUTTON_TOOLBAR);
        but->setIconPosition (ICON_BEFORE_TEXT);
        but->setJustify (JUSTIFY_LEFT);
	
		vector<string> shutterCommands;
		
		if(*iter==defaultShutter)
		defaultShutterNum=counter;
		
		vector<string>::iterator iter2;	
		if(*iter!="special")
		{
			shutterCommands=fileList->filterContexCommands(settings.getShutter(*iter));
				
		}
		else
		{
			shutterCommands=fileList->getAvailableCommandsForSelectedFiles();
		}
		
		for(iter2=shutterCommands.begin();iter2!=shutterCommands.end();++iter2)
		{			
			FXButton* button=new FXButton (shutterItem->getContent (),settings.getCommand(*iter2).getDescription(settings.getLanguage()).c_str(), foxIconManager.get("symlink"), this, settings.getCommand(*iter2).getNr(), FRAME_RAISED | ICON_BEFORE_TEXT | JUSTIFY_LEFT | LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT, 0, 0, 0, 0, 0, 0, 0, 0);
			
			//if(*iter=="special")
			//cout<<"C="<<*iter2<<endl;
			
			int labelWidth=button->getFont()->getTextWidth(settings.getCommand(*iter2).getDescription(settings.getLanguage()).c_str())+button->getIcon()->getWidth();
			if(labelWidth>maxLabelWidth)
			maxLabelWidth=labelWidth;
		}
	++counter;	
	}
	
	foxShutter->setWidth(maxLabelWidth);
	foxShutter->setCurrent(defaultShutterNum);
	
	FXEvent *event = (FXEvent *) ptr;
	int popup_x = event->root_x - 5;
    	int popup_y = event->root_y - 5;


	int heightSize= ((FoxShutter *) foxShutter)->getHeight ();
    	popupMenu->create ();
	popupMenu->popup (NULL, popup_x, popup_y, maxLabelWidth+10, heightSize);
	getApp ()->runModalWhileShown (popupMenu);
	return 1;
	
}

long FoxFileList::onSort( FXObject * obj, FXSelector sel, void * ptr )
{

    FXushort id = FXSELID (sel);
    int num;

    if (id == ID_SORT_CHANGE)
    {	
	num = sortMenu->getCurrentNo ();	
    }
    else
    {
    num=(FXuint)(FXuval)ptr;
    }
    
	fileList->getFileListSync()->sort(num);
	
	return 1;
}



long FoxFileList::onCmdResizeHeader( FXObject * obj, FXSelector sel, void * ptr )
{
	vector<int>vec;
	for(int i=0;i<getNumHeaders();i++)
	{
	vec.push_back(getHeaderSize(i));
	}	
	settings.setHeadersWidth(fileList->getVfsName(), vec);
	return 1;
}


long FoxFileList::onMouseButtonPress( FXObject * obj, FXSelector sel, void * ptr )
{
	mousePressed=true;	
	FXIconList::onLeftBtnPress(obj,sel,ptr);
	return 1;

}



long FoxFileList::onMouseButtonRelease( FXObject * obj, FXSelector sel, void * ptr )
{
	mousePressed=false;
	FXIconList::onLeftBtnRelease(obj,sel,ptr);
	fileList->clicked();
	return 1;
}


long FoxFileList::onFocusNext( FXObject * obj, FXSelector sel, void * ptr )
{
	FXEvent* event=(FXEvent*)ptr;

	switch(MKUINT(event->code,event->state&(SHIFTMASK|CONTROLMASK|ALTMASK|METAMASK)))
	{
    	case KEY_Tab:
    		handle(this,FXSEL(SEL_COMMAND,tabCommandNr ),NULL);
    		break;
    	default:
    		FXIconList::onKeyPress(obj,sel,ptr);
    	}

	
}

long FoxFileList::onTimer(FXObject * obj, FXSelector sel, void * ptr)
{
	++delay;
	
	if(delay==4 &&lastAnyFileSelected!=isFileIconSelectedSync())
	{
		lastAnyFileSelected=isFileIconSelectedSync();
		if(active)
		showToolbars();	
	}
	
	if(delay==4)
	delay=0;

	getApp () ->addTimeout ( this, ID_TIMER, 200 );	
	
	vector<FoxStatus*>::iterator iter;
	
	fileList->tick();
	
	for(iter=statusVector.begin();iter!=statusVector.end();++iter)
	{
		if(!(*iter)->isClosed())
		{
			(*iter)->tick();
		}
		else
		{				
			delete *iter;
			statusVector.erase(iter);
			break;
		}
	}
	
	if(processing)
	{
		
		executionButton->setIcon(foxIconManager.get("execute"+lexical_cast<string>(rotateAngel)));
	
		if(rotateAngel==2)
		{
			rotateAngel=1;
		}
		else
			++rotateAngel;				
		
	}
	else
	{
	executionButton->setIcon(NULL);
	}
	
return 1;	
}

long FoxFileList::onOpenFile( FXObject * obj, FXSelector sel, void * ptr )
{
	int k = getCurrentItem ();
	if(k==-1)
	return 0;
	FoxFileListItem* listItem=dynamic_cast<FoxFileListItem*>(getItem (k));
	fileList->openIcon(listItem->getFileIcon());
	return 1;
}

void FoxFileList::replaceFileIcon(int position, FileIcon fileIcon)
{
	prepare<void>(boost::bind(&FoxFileList::replaceFileIconSync, this,position,fileIcon),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);

}

FileIcon FoxFileList::getFileIcon(int position )
{
	return prepare<FileIcon>(boost::bind(&FoxFileList::getFileIconSync, this,position),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL_FILEICON,mutexS);

}

void FoxFileList::replaceFileIconSync(int position, FileIcon fileIcon )
{
	if(position<getNumItems())
	{	
		FXIcon *icon;
		
		if(fileIcon.getPreviewIcon()!=-1)
		{
			icon=previewIcons[fileIcon.getPreviewIcon()];
			icon->create();
		}
		else
		{
		icon=foxIconManager.get(fileIcon.getIconName());
		}
		
		FXIcon *extraIcon=NULL;
		if(fileIcon.getExtraIconName().size())
		{
			extraIcon=foxIconManager.get(fileIcon.getExtraIconName());
		}	
		
		FoxFileListItem *listItem=new FoxFileListItem(fileIcon,icon,icon,app,extraIcon);
		setItem (position,listItem);
	}
	

}

FileIcon FoxFileList::getFileIconSync(int position )
{
	if(position<getNumItems())
	{
	FoxFileListItem *listItem = dynamic_cast<FoxFileListItem *>( getItem (position) );
	return listItem->getFileIcon();
	}
	else
	throw exception();
}

int FoxFileList::loadPreviewIcon(string file, int size)
{
	FXString name=file.c_str();
		
			FXIcon *icon=NULL;
			#ifdef  HAVE_LIBFOX_1_6
			icon=source->loadScaledIconFile ( name, size );
			#else
			icon=source->loadScaledIcon ( name, size );	
			#endif		
	
			if(icon && (icon->getWidth()<8 || icon->getHeight()<8))
			{
			delete icon;
			icon=NULL;
			}
	
			if ( icon )
			{
				{
				boost::mutex::scoped_lock lock(muteX);
				previewIcons.push_back(icon);
				}
				icon->blend(icon->getTransparentColor());
				//1icon->create();
				return previewIcons.size()-1;
			}
			
			return -1;
}





void FoxFileList::unloadPreviewIcons()
{
	boost::mutex::scoped_lock lock(muteX);

	std::vector<FXIcon*>::iterator iter;
	for(iter=previewIcons.begin();iter!=previewIcons.end();++iter)
	{
		delete *iter;
	}
	previewIcons.clear();

}





void FoxFileList::disable()
{
	prepare<void>(boost::bind(&FoxFileList::disableSync, this),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);
}

void FoxFileList::disableSync()
{
	//FXIconList::disable();
	panel->disable();
}

void FoxFileList::enable()
{
	prepare<void>(boost::bind(&FoxFileList::enableSync, this),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);
}

void FoxFileList::enableSync()
{
	//FXIconList::enable();
	panel->enable();
}

long FoxFileList::onStopExecution(FXObject * obj, FXSelector sel, void * ptr)
{
	executionButton->disable();
	fileList->runCommand("StopThreads");
	
}

bool FoxFileList::isProcessing()
{	
	if(processing)
	return true;
	else
	{
		if(fileList->isAcquired())
		return true;
		else
		return false;
	}

}






void FoxFileList::createPanel(string name, string param)
{
	prepare<void>(boost::bind(&FoxFileList::createPanelSync, this,name,param),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);
	
}

void FoxFileList::createPanelSync(string name, string param)
{
	panel->createPanel(name,param);
}

void FoxFileList::ask(WindowContent * windowContent, map< string, string > & result)
{
	
	
	boost::mutex m_mutex;
	boost::condition m_resultReadyCondition;    	
	bool m_resultReady=false;
	
	Wait w;
	w.m_mutex=&m_mutex;
	w.m_resultReadyCondition=&m_resultReadyCondition;
	w.m_resultReady=&m_resultReady; 
	w.result=&result;	
	
	prepare<void>(boost::bind(&FoxFileList::askSync, this, &w,windowContent),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);
	
	boost::mutex::scoped_lock lock(m_mutex);
	if(m_resultReady==false)		
	m_resultReadyCondition.wait(lock);
	
	
}

void FoxFileList::askSync(Wait *w,WindowContent * windowContent)
{
	
	w->foxInputDialog=new FoxInputDialog(this,w,windowContent);
	w->foxInputDialog->create();
	w->foxInputDialog->show(PLACEMENT_CURSOR);
}

void FoxFileList::removeFileIcon(int position)
{
	prepare<void>(boost::bind(&FoxFileList::removeFileIconSync, this,position),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);
}

void FoxFileList::removeFileIconSync(int position)
{
	if(position<getNumItems())
	removeItem(position);
}


void FoxFileList::selectFileIcon(int position)
{
	prepare<void>(boost::bind(&FoxFileList::selectFileIconSync, this,position),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);

}

void FoxFileList::selectFileIconSync(int position)
{
	if(position<getNumItems())
	selectItem(position);
}

FileIcon FoxFileList::getCurrentFileIcon()
{
return prepare<FileIcon>(boost::bind(&FoxFileList::getCurrentFileIconSync, this),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL_FILEICON,mutexS);
}

FileIcon FoxFileList::getCurrentFileIconSync()
{
	if(getCurrentItem()!=-1)
	{	
	FoxFileListItem* listItem=dynamic_cast<FoxFileListItem*>(getItem(getCurrentItem()));
	return listItem->getFileIcon(); 
	}
	else
	throw exception();
}

long FoxFileList::onRightButtonPress(FXObject *, FXSelector , void *ptr)
{
	pressedRightButton=true;
	FXEvent *event=static_cast<FXEvent*>(ptr);
	lastX=event->root_x;
	lastY=event->root_y;
	
}

long FoxFileList::onMotion( FXObject * obj, FXSelector sel, void * ptr )
{
	FXIconList::onMotion(obj,sel,ptr);
	
	FXEvent *event=static_cast<FXEvent*>(ptr);
	
	if(pressedRightButton)
	{

		
		int min=10;
		
		int PX = event->root_x - lastX; 
   		int PY = event->root_y - lastY; 
    
    		int absPX = abs(PX); 
    		int absPY = abs(PY);
    		
    		int Gest;
    
		if ((absPX > min) || (absPY > min)) 
		{
			if ((absPX > min) && (absPY < min*2)) 
			{
				if (PX < 0)
				Gest = 4; // left
				else
				Gest = 6; // right
			} 
			else if ((absPY > min) && (absPX < min*2)) 
			{
				if (PY < 0)
				Gest = 8; //up
				else
				Gest = 2; //down
			} 
			else 
			{
				gests.clear();
				mouseGestureActive=false;
			}
			
			mouseGestureActive=true;
			

			if(gests.size()==0 || gests[gests.size()-1]!=Gest)
			gests.push_back(Gest); 
			
			lastX = event->root_x; 
			lastY = event->root_y;
				
		}
	}
	
}

void FoxFileList::updateTextFieldList(void )
{

	if(textField)
	{
		textField->clearItems ();
	

		vector< string >::iterator iter;
		for(iter=textFieldList.begin();iter!=textFieldList.end();++iter)
		{
				textField->appendItem(iter->c_str());
		}
		int num=textFieldList.size()<25?textFieldList.size():25;
		textField->setNumVisible(num);
		textField->recalc();
	}

}


