/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef FOXGUITHREADSYNCHRONIZATION_HPP
#define FOXGUITHREADSYNCHRONIZATION_HPP

/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/


#include "config.h"

#include <fx.h>

#ifdef  HAVE_LIBFOX_1_4
#include <FXGUISignal.h>
#endif


#include <boost/function.hpp>
#include <boost/bind.hpp>
#include <boost/thread/mutex.hpp>
#include <boost/thread/condition.hpp>

#include <iostream>
#include <map>
using namespace std;

template<typename T>
class tt
{
	public:
	boost::mutex *m_mutex;
	boost::condition *m_resultReadyCondition;
	bool *m_resultReady;
	boost::function<T (void)> fun;
	T returnValue;
};

template<>
class tt<void>
{
	public:
	boost::mutex *m_mutex;
	boost::condition *m_resultReadyCondition;
	bool *m_resultReady;
	boost::function<void (void)> fun;
};


class Wait
{
	public:
	boost::mutex *m_mutex;
	boost::condition *m_resultReadyCondition;
	bool *m_resultReady;
	map<string,string> *result;
	FXDialogBox *foxInputDialog;
};



namespace
{

template<typename T>
T prepare(boost::function<T (void)> fun,pthread_t p_id, FXGUISignal *guiSignal,FXSelector sel,boost::mutex &mutexS)
{

	pthread_t id=pthread_self();
	if(p_id==id)
    		return fun();    	
    	else
    	{    	
    		boost::mutex::scoped_lock lock0(mutexS);
    	
		boost::mutex m_mutex;
		boost::condition m_resultReadyCondition;    	
		bool m_resultReady=false;
						
		tt<T> t;
		t.m_mutex=&m_mutex;
		t.m_resultReadyCondition=&m_resultReadyCondition;
		t.m_resultReady=&m_resultReady;
		t.fun=fun;
		
						
		guiSignal->setData((void*)&t);
		guiSignal->setSelector(sel);
		guiSignal->signal(); 
		
		boost::mutex::scoped_lock lock(m_mutex);
		if(m_resultReady==false)
		m_resultReadyCondition.wait(lock);		
		return t.returnValue;
         }
}

template<>
void prepare<void>(boost::function<void (void)> fun,pthread_t p_id, FXGUISignal *guiSignal,FXSelector sel,boost::mutex &mutexS)
{

	pthread_t id=pthread_self();
	if(p_id==id)
    		fun();    	
    	else
    	{    	
    		boost::mutex::scoped_lock lock0(mutexS);
    	
		boost::mutex m_mutex;
		boost::condition m_resultReadyCondition;    	
		bool m_resultReady=false;
						
		tt<void> t;
		t.m_mutex=&m_mutex;
		t.m_resultReadyCondition=&m_resultReadyCondition;
		t.m_resultReady=&m_resultReady;
		t.fun=fun;
						
		guiSignal->setData((void*)&t);
		guiSignal->setSelector(sel);
		guiSignal->signal(); 
		
		boost::mutex::scoped_lock lock(m_mutex);
	
		if(m_resultReady==false)		
		m_resultReadyCondition.wait(lock);
         }
}


template<typename T>
long onGuiSignalX(FXObject * obj, FXSelector sel, void * ptr)
{
	tt<T> *t=(tt<T>*)ptr;
	t->returnValue=t->fun();	
	boost::mutex::scoped_lock lock(*t->m_mutex);
	(*t->m_resultReady)=true;
	(*t->m_resultReadyCondition).notify_one();
}

template<>
long onGuiSignalX<void>(FXObject * obj, FXSelector sel, void * ptr)
{
	tt<void> *t=(tt<void>*)ptr;
	t->fun();
	
	boost::mutex::scoped_lock lock(*t->m_mutex);
	(*t->m_resultReady)=true;
	(*t->m_resultReadyCondition).notify_one();
}

}



class FoxSingalReceiver: public FXObject
{
FXDECLARE(FoxSingalReceiver);
public:

template<typename T>
	long onGuiSignal ( FXObject * obj, FXSelector sel, void *ptr )
	{
		::onGuiSignalX<T>(obj,sel,ptr);
	}

	enum
	{
	ID_GUISIGNAL,
	ID_GUISIGNAL_STRING,
     	ID_GUISIGNAL_BOOL,
     	ID_GUISIGNAL_FILEICON,
     	ID_GUISIGNAL_VECTOR_FILEICON,
     	ID_GUISIGNAL_INT,
        ID_LAST,
	};


};







#endif
