/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef VFSARCHIVE_HPP
#define VFSARCHIVE_HPP

#include "../Vfs.hpp"
#include "../Mime.hpp"

#include "../libfactory/ClassFactory.hpp"
#include "../libfactory/Factorable.hpp"



/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/
class VfsArchive : public Vfs, public Factorable
{
public:
	VfsArchive();
	~VfsArchive();
    	Directory openDir( path &p, Cancelable *cancelable=NULL );    
	File readFile(path file);
	void createDir(path p);
	
	void copyFromFile(File &source, path &destination, Status * status, Cancelable *cancelable);
	void copyToFile(File &source, path &destination, Status * status, Cancelable *cancelable);
	void copy(Vfs * vfs2,path source, path destination,Status * status, Cancelable *cancelable);
	
	path symlink(const path& file);    
	double getTotalSize(path &p, Cancelable *cancelable);
	string getType();
	void init(Panel *panel,QuestionManager *questionManager);
	void chmod(path &source,unsigned int mode);
	void chmodRecursive(path &source,unsigned int mode);
	void removeFile(path &source);
    	void removeDirectory(path &source);
    	void remove(path &source);
    	void rename(path &source, path &destination);
    	File readFullFile(File file);
    	void changeOwnerRecursive(path &source,string user);
    	void changeOwner(path &source,string user);
    	void changeGroupRecursive(path &source,string group);
    	void changeGroup(path &source,string group);
    	vector<string> getUsers();
    	vector<string> getGroups();
    	void createSymlink(path &source,path &destination);
    
private:

	File readDir( Directory *directory);  
	void closeDir( Directory *directory);  
	void listFiles();
	path createTmpDir();
	void compressArchive();
	void decompressArchive();
	
	
	vector<File>allFiles;
	vector<File>::iterator iter;
	map<string,string>params;
	QuestionManager *questionManager;
	Panel * panel;
	bool listed;
	string type;
	FILE *pipe;
	Mime mime;
	string archiveFileName;
	string decompress,compress,archiveFilenameDecompressed;
};

#endif
