/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "VfsSearch.hpp"
#include "FilesEngine.hpp"

#include <boost/regex.hpp>

#include "config.h"
#include "gettext.h"
#define _(STRING)    gettext(STRING)



REGISTER_FACTORABLE(VfsSearch);


class OperationSearch
{
public:	
	
	OperationSearch(vector<File> &_files, boost::regex &_pattern):files(_files),pattern(_pattern)
	{
	}
	
	void process(File file)
	{
		//if(file.getSize()<50000)
		//{
		//	files.push_back(file);
		//}
		
		if(boost::regex_match(file.getName(), pattern))
		{
			files.push_back(file);
		}
	}

private:
	vector<File>& files;
	boost::regex &pattern;
	
};


VfsSearch::VfsSearch() : initialized(false)
{
}


VfsSearch::~VfsSearch()
{
}

Directory VfsSearch::openDir(path & p, Cancelable *cancelable)
{
	string searchDir=params["directory"];
	

	if(p.string()!=searchDir)
	{
 		panel->createPanel("FileList","vfs=Local:directory="+p.string());
	}

	path pp=path(searchDir);

	if(!initialized)
	{
		files.clear();

		try
		{
		boost::regex pattern(params["filePattern"]); 			
		
		
		OperationSearch operation(files,pattern);
		File file=readFile(pp);
		VfsLocal vfsLocal;
		vfsLocal.goRecursive(file,operation,false,cancelable);  
		
		}
		catch(boost::bad_expression &e){}
		initialized=true;
	}
	iter=files.begin(); 
	Directory directory(this,NULL,p);
	return directory;
}


File VfsSearch::readDir(Directory * directory)
{
	if(iter!=files.end())
	return *iter++;
	else
	throw ExceptionReadDirEnd();
}

void VfsSearch::init(Panel * _panel, QuestionManager * _questionManager)
{
	VfsBase::init(_panel,_questionManager);

	params=FilesEngine::parseParam(panel->getParam());
	
	panel->setLabel(_("search"),"search_small:"+params["directory"]);
	setHeaders(settings.getHeaders("Search"));	
}



